@@ -1,8 +1,6 @@
 from dataclasses import dataclass
 from dataclasses import field
 
-PROJECT_ROOT_FILE = 'project.yaml'
-
 
 @dataclass
 class Target:
@@ -34,9 +32,11 @@ class Project(object):
         self.name = name
         self.version = "0.1"
         self.tests = []
-        self.plugins = []
+        self.declared_bits = {}
+        self.bits = []
         self.sources = []
         self.packages = []
+        self.compile_flags = []
         self.include_directories = []
         self.link_options = LinkOptions()
         self.actions = []
@@ -45,8 +45,8 @@ class Project(object):
     def add_target(self, target):
         self.targets[target.name] = target
 
-    def add_plugin(self, plugin):
-        self.plugins.append(plugin)
+    def add_bit(self, bit):
+        self.bits.append(bit)
 
     def add_sources(self, sources):
         self.sources.extend(sources)
@@ -64,4 +64,7 @@ class Project(object):
         self.link_options.libraries.append(library)
 
     def add_action(self, project_action):
-        self.actions.append(project_action)
\ No newline at end of file
+        self.actions.append(project_action)
+
+    def add_compile_flags(self, compile_flags):
+        self.compile_flags.extend(compile_flags)
