@@ -6,7 +6,7 @@ from mock import patch
 
 from subprocess import CompletedProcess
 from cpm.domain.cmake_recipe import CMakeRecipe, TestsFailed, BUILD_DIRECTORY, CMAKELISTS, CompilationError
-from cpm.domain.plugin import Plugin
+from cpm.domain.bit import Bit
 from cpm.domain.project import Project, Package
 
 
@@ -15,7 +15,7 @@ class TestCMakeRecipe(unittest.TestCase):
         filesystem = MagicMock()
         CMakeRecipe(filesystem)
 
-    def test_recipe_generation_without_plugins_or_packages(self):
+    def test_recipe_generation_without_bits_or_packages(self):
         filesystem = self.filesystemMockWithoutRecipeFiles()
         project = _project_without_sources('DeathStarBackend')
         cmake_recipe = CMakeRecipe(filesystem)
@@ -30,11 +30,6 @@ class TestCMakeRecipe(unittest.TestCase):
             'project(DeathStarBackend)\n'
             'include_directories()\n'
             'add_executable(DeathStarBackend main.cpp)\n'
-            'add_custom_command(\n'
-            '    TARGET DeathStarBackend\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:DeathStarBackend> ${PROJECT_SOURCE_DIR}/DeathStarBackend\n'
-            ')\n'
         )
 
     def test_recipe_generation_with_target_link_libraries(self):
@@ -55,11 +50,6 @@ class TestCMakeRecipe(unittest.TestCase):
             'include_directories()\n'
             'add_executable(DeathStarBackend main.cpp)\n'
             'target_link_libraries(DeathStarBackend pthread rt)\n'
-            'add_custom_command(\n'
-            '    TARGET DeathStarBackend\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:DeathStarBackend> ${PROJECT_SOURCE_DIR}/DeathStarBackend\n'
-            ')\n'
         )
 
     def test_recipe_generation_with_one_package(self):
@@ -79,11 +69,27 @@ class TestCMakeRecipe(unittest.TestCase):
             'project(DeathStarBackend)\n'
             'include_directories(package)\n'
             'add_executable(DeathStarBackend main.cpp)\n'
-            'add_custom_command(\n'
-            '    TARGET DeathStarBackend\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:DeathStarBackend> ${PROJECT_SOURCE_DIR}/DeathStarBackend\n'
-            ')\n'
+        )
+
+    def test_recipe_generation_with_one_package_with_global_compile_flags(self):
+        filesystem = self.filesystemMockWithoutRecipeFiles()
+        project = _project_without_sources('DeathStarBackend')
+        project.add_package(Package('package', sources=['package.cpp']))
+        project.add_include_directory('package')
+        project.add_sources(['package.cpp'])
+        project.compile_flags = ['-g']
+        cmake_recipe = CMakeRecipe(filesystem)
+
+        cmake_recipe.generate(project)
+
+        filesystem.create_file.assert_called_once_with(
+            'CMakeLists.txt',
+
+            'cmake_minimum_required (VERSION 3.7)\n'
+            'project(DeathStarBackend)\n'
+            'include_directories(package)\n'
+            'add_executable(DeathStarBackend main.cpp package.cpp)\n'
+            'set_target_properties(DeathStarBackend PROPERTIES COMPILE_FLAGS "-g")\n'
         )
 
     def test_recipe_generation_with_one_package_with_cflags(self):
@@ -104,11 +110,6 @@ class TestCMakeRecipe(unittest.TestCase):
             'include_directories(package)\n'
             'set_source_files_properties(package.cpp PROPERTIES COMPILE_FLAGS "-std=c++11 -DMACRO")\n'
             'add_executable(DeathStarBackend main.cpp package.cpp)\n'
-            'add_custom_command(\n'
-            '    TARGET DeathStarBackend\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:DeathStarBackend> ${PROJECT_SOURCE_DIR}/DeathStarBackend\n'
-            ')\n'
         )
 
     def test_recipe_generation_with_many_packages_with_cflags(self):
@@ -132,16 +133,30 @@ class TestCMakeRecipe(unittest.TestCase):
             'set_source_files_properties(package1.cpp PROPERTIES COMPILE_FLAGS "-std=c++11")\n'
             'set_source_files_properties(package2.cpp PROPERTIES COMPILE_FLAGS "-Wall")\n'
             'add_executable(DeathStarBackend main.cpp package1.cpp package2.cpp)\n'
-            'add_custom_command(\n'
-            '    TARGET DeathStarBackend\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:DeathStarBackend> ${PROJECT_SOURCE_DIR}/DeathStarBackend\n'
-            ')\n'
         )
 
-    def test_recipe_generation_with_one_plugin(self):
+    def test_recipe_generation_with_one_bit(self):
         filesystem = self.filesystemMockWithoutRecipeFiles()
-        project = _project_with_one_plugin('DeathStarBackend')
+        project = _project_with_one_bit('DeathStarBackend', sources=['bit.cpp'])
+        cmake_recipe = CMakeRecipe(filesystem)
+
+        cmake_recipe.generate(project)
+
+        filesystem.create_directory.assert_called_once_with('build')
+        filesystem.create_file.assert_called_once_with(
+            'CMakeLists.txt',
+
+            'cmake_minimum_required (VERSION 3.7)\n'
+            'project(DeathStarBackend)\n'
+            'include_directories()\n'
+            'add_library(cest STATIC bit.cpp)\n'
+            'add_executable(DeathStarBackend main.cpp)\n'
+            'target_link_libraries(DeathStarBackend cest)\n'
+        )
+
+    def test_recipe_generation_with_one_bit_without_sources(self):
+        filesystem = self.filesystemMockWithoutRecipeFiles()
+        project = _project_with_one_bit('DeathStarBackend')
         cmake_recipe = CMakeRecipe(filesystem)
 
         cmake_recipe.generate(project)
@@ -154,16 +169,11 @@ class TestCMakeRecipe(unittest.TestCase):
             'project(DeathStarBackend)\n'
             'include_directories()\n'
             'add_executable(DeathStarBackend main.cpp)\n'
-            'add_custom_command(\n'
-            '    TARGET DeathStarBackend\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:DeathStarBackend> ${PROJECT_SOURCE_DIR}/DeathStarBackend\n'
-            ')\n'
         )
 
-    def test_recipe_generation_with_one_plugin_with_a_package_with_cflags(self):
+    def test_recipe_generation_with_one_bit_with_a_package_with_cflags(self):
         filesystem = self.filesystemMockWithoutRecipeFiles()
-        project = _project_with_one_plugin('DeathStarBackend', sources=['plugins.cpp'], cflags=['-DDEFINE'])
+        project = _project_with_one_bit('DeathStarBackend', sources=['bit.cpp'], cflags=['-DDEFINE'])
         cmake_recipe = CMakeRecipe(filesystem)
 
         cmake_recipe.generate(project)
@@ -175,13 +185,31 @@ class TestCMakeRecipe(unittest.TestCase):
             'cmake_minimum_required (VERSION 3.7)\n'
             'project(DeathStarBackend)\n'
             'include_directories()\n'
-            'set_source_files_properties(plugins.cpp PROPERTIES COMPILE_FLAGS "-DDEFINE")\n'
+            'add_library(cest STATIC bit.cpp)\n'
+            'set_source_files_properties(bit.cpp PROPERTIES COMPILE_FLAGS "-DDEFINE")\n'
             'add_executable(DeathStarBackend main.cpp)\n'
-            'add_custom_command(\n'
-            '    TARGET DeathStarBackend\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:DeathStarBackend> ${PROJECT_SOURCE_DIR}/DeathStarBackend\n'
-            ')\n'
+            'target_link_libraries(DeathStarBackend cest)\n'
+        )
+
+    def test_recipe_generation_with_one_bit_and_link_libraries(self):
+        filesystem = self.filesystemMockWithoutRecipeFiles()
+        project = _project_with_one_bit('DeathStarBackend', sources=['bit.cpp'], cflags=['-DDEFINE'])
+        project.add_library('boost')
+        cmake_recipe = CMakeRecipe(filesystem)
+
+        cmake_recipe.generate(project)
+
+        filesystem.create_directory.assert_called_once_with('build')
+        filesystem.create_file.assert_called_once_with(
+            'CMakeLists.txt',
+
+            'cmake_minimum_required (VERSION 3.7)\n'
+            'project(DeathStarBackend)\n'
+            'include_directories()\n'
+            'add_library(cest STATIC bit.cpp)\n'
+            'set_source_files_properties(bit.cpp PROPERTIES COMPILE_FLAGS "-DDEFINE")\n'
+            'add_executable(DeathStarBackend main.cpp)\n'
+            'target_link_libraries(DeathStarBackend cest boost)\n'
         )
 
     def test_recipe_generation_with_one_test_suite(self):
@@ -200,11 +228,6 @@ class TestCMakeRecipe(unittest.TestCase):
             'project(DeathStarBackend)\n'
             'include_directories()\n'
             'add_executable(DeathStarBackend main.cpp source.cpp)\n'
-            'add_custom_command(\n'
-            '    TARGET DeathStarBackend\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:DeathStarBackend> ${PROJECT_SOURCE_DIR}/DeathStarBackend\n'
-            ')\n'
             'add_library(DeathStarBackend_object_library OBJECT source.cpp)\n'
             'add_executable(test_suite tests/test_suite.cpp $<TARGET_OBJECTS:DeathStarBackend_object_library>)\n'
             'set_target_properties(test_suite PROPERTIES COMPILE_FLAGS "-std=c++11 -g")\n'
@@ -214,6 +237,34 @@ class TestCMakeRecipe(unittest.TestCase):
             ')\n'
         )
 
+    def test_recipe_generation_with_one_test_suite_and_one_bit_and_link_libraries(self):
+        filesystem = self.filesystemMockWithoutRecipeFiles()
+        project = _project_with_one_bit('DeathStarBackend', ['bit.cpp'])
+        project.tests = ['tests/test_suite.cpp']
+        project.add_library('boost')
+        cmake_recipe = CMakeRecipe(filesystem)
+
+        cmake_recipe.generate(project)
+
+        filesystem.create_directory.assert_called_once_with('build')
+        filesystem.create_file.assert_called_once_with(
+            'CMakeLists.txt',
+
+            'cmake_minimum_required (VERSION 3.7)\n'
+            'project(DeathStarBackend)\n'
+            'include_directories()\n'
+            'add_library(cest STATIC bit.cpp)\n'
+            'add_executable(DeathStarBackend main.cpp)\n'
+            'target_link_libraries(DeathStarBackend cest boost)\n'
+            'add_executable(test_suite tests/test_suite.cpp)\n'
+            'set_target_properties(test_suite PROPERTIES COMPILE_FLAGS "-std=c++11 -g")\n'
+            'target_link_libraries(test_suite cest boost)\n'
+            'add_custom_target(test\n'
+            '    COMMAND echo "> Done"\n'
+            '    DEPENDS test_suite\n'
+            ')\n'
+        )
+
     def test_recipe_generation_with_many_test_suites(self):
         filesystem = self.filesystemMockWithoutRecipeFiles()
         project = _project_with_sources('DeathStarBackend', ['source.cpp'])
@@ -230,11 +281,6 @@ class TestCMakeRecipe(unittest.TestCase):
             'project(DeathStarBackend)\n'
             'include_directories()\n'
             'add_executable(DeathStarBackend main.cpp source.cpp)\n'
-            'add_custom_command(\n'
-            '    TARGET DeathStarBackend\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:DeathStarBackend> ${PROJECT_SOURCE_DIR}/DeathStarBackend\n'
-            ')\n'
             'add_library(DeathStarBackend_object_library OBJECT source.cpp)\n'
             'add_executable(test_suite_1 tests/test_suite_1.cpp $<TARGET_OBJECTS:DeathStarBackend_object_library>)\n'
             'set_target_properties(test_suite_1 PROPERTIES COMPILE_FLAGS "-std=c++11 -g")\n'
@@ -265,11 +311,6 @@ class TestCMakeRecipe(unittest.TestCase):
             'include_directories()\n'
             'add_executable(DeathStarBackend main.cpp source.cpp)\n'
             'target_link_libraries(DeathStarBackend pthread rt)\n'
-            'add_custom_command(\n'
-            '    TARGET DeathStarBackend\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:DeathStarBackend> ${PROJECT_SOURCE_DIR}/DeathStarBackend\n'
-            ')\n'
             'add_library(DeathStarBackend_object_library OBJECT source.cpp)\n'
             'add_executable(test_suite tests/test_suite.cpp $<TARGET_OBJECTS:DeathStarBackend_object_library>)\n'
             'set_target_properties(test_suite PROPERTIES COMPILE_FLAGS "-std=c++11 -g")\n'
@@ -296,11 +337,6 @@ class TestCMakeRecipe(unittest.TestCase):
             'project(Cest)\n'
             'include_directories()\n'
             'add_executable(Cest main.cpp)\n'
-            'add_custom_command(\n'
-            '    TARGET Cest\n'
-            '    POST_BUILD\n'
-            '    COMMAND COMMAND ${CMAKE_COMMAND} -E copy $<TARGET_FILE:Cest> ${PROJECT_SOURCE_DIR}/Cest\n'
-            ')\n'
             'add_executable(test_cest tests/test_cest.cpp)\n'
             'set_target_properties(test_cest PROPERTIES COMPILE_FLAGS "-std=c++11 -g")\n'
             'add_custom_target(test\n'
@@ -503,14 +539,13 @@ def _project_without_sources(name):
     return project
 
 
-def _project_with_one_plugin(name, sources=[], cflags=[]):
+def _project_with_one_bit(name, sources=[], cflags=[]):
     project = Project(name)
     project.sources = ['main.cpp']
-    plugin = Plugin('cest')
-    plugin.add_package(Package('plugins/cest', sources=sources, cflags=cflags))
-    project.add_plugin(Plugin('cest'))
-    for package in plugin.packages:
-        project.add_package(package)
+    bit = Bit('cest')
+    bit.add_package(Package('bits/cest', sources=sources, cflags=cflags))
+    bit.add_sources(sources)
+    project.add_bit(bit)
     return project
 
 
